/* config.cpp -- command line parsing functions

   This file is part of the SIB - simple IPX bridge

   Copyright (C) 2001 Szomraky Stefan

   The SIB is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of
   the License, or (at your option) any later version.

   The SIB is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with the SIB; see the file COPYING.
   If not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
   
   Szomraky Stefan
   stsz@gmx.net
 */
#include "config.h"

/** parseConfig( int argc, char **argv)
  
parses commandline (maybe it will parse a config file too) and fills out
global "Config" structure named config

Arguments: argc and argv of main

**/
int parseConfig( int argc, char **argv)
{
 Config lc;
 int d;
 
 // set to standard values
 memset( &lc, 0, sizeof(lc));
 // now everything is set to 0 - set 1s
 lc.doPromisc = 1;
 lc.noIPXSMB = 1;
 lc.protoMask.IPX = 1; 
 
 // set default MTU
 lc.doMTU = 1;
 lc.MTU=DEFAULT_MTU;
 lc.MTUseq=255;
 
 while( (d = getopt( argc, argv, "acC:d::f:hH:i:kmp:qst:v:")) > 0) {
  switch( d) {
   case ':': 
   case '?':
   case 'h':
    printHelp();
    return -1;
    
   case 'a':
    lc.noMACFilter = 1;
    break;

#ifdef HAVE_LZO   
   case 'c':
    lc.doCompression = 1;
    break;
#else
   case 'c':
    printGreet();
    fprintf( stderr, "ERROR: no compression support built in\n");
    return -1;
#endif
   case 'C':
    lc.custlib = dlopen( optarg, RTLD_NOW);
    if( lc.custlib == NULL) {
     perror( "dlopen");
     printf( "%s\n", dlerror());
     return -1;
    }
    lc.customFilter = (filter_func) dlsym( lc.custlib, "filterFrame");
    if( lc.customFilter == NULL) {
     perror( "dlsym");
     return -1;
    } 
    break;
    
   case 'd':
    if( optarg != NULL)
     lc.debug = atoi(optarg);
    else
     lc.debug = 1;
    break;
    
   case 'f':
    if( setMask( &lc, optarg, 0) < 0) {
     printGreet();
     printf( "ERROR: %s is not a legal protocol type\n", optarg);
     return -1;
    } 
    break;
          
   case 'H':
    lc.remoteHost = strdup( optarg);
    break;
   
   case 'i':
    lc.interfaceName = strdup( optarg);
    break;
    
   case 'k':
    lc.doPromisc = 0;
    break;

   case 'm':
    lc.doMTU = 0;
    break;
    
   case 'p':
    lc.udpPort = atoi( optarg);
    break;
    
   case 'q':
    lc.quiet = 1;
    break;
    
   case 's':
    lc.noIPXSMB = 0;
    break;
    
   case 't':
    if( setMask( &lc, optarg, 1) < 0) {
     printGreet();
     printf( "ERROR: %s is not a legal protocol type\n", optarg);
     return -1;
    }
    break;
     
   case 'v':
    lc.statsInterval = atoi( optarg);
    break;                       
    
   default:
    printHelp();       
  }
 }
 
 // at least those must be set
 if( !lc.remoteHost || !lc.udpPort || !lc.interfaceName) {
  printHelp();
  return -1;
 }
 
 if( lc.debug) {
  if( lc.protoMask.IP) printf( "IP is set\n");
  if( lc.protoMask.IPX) printf( "IPX is set\n");
 }
 
 // protomask must not be empty)
 if( !*(u_int *)&(lc.protoMask))  {
  printf( "ERROR: all protocols filtered! see %s -h for details\n\n", PROGNAME);
  return -1;
 }

 // had problems with port 4000 on masquerading hosts
 if( config.udpPort == 4000)
  printf( "WARNING: port 4000 (UDP) is used by ICQ and may lead to missbehaviour\n");
 
 // we're ready, copy lc to config
 memcpy( &config, &lc, sizeof( lc));
 return 0;
}

int setMask( Config *c, char *proto, int state)
{
 u_int *pInt = (u_int *)&(c->protoMask);
 switch( int i = atoProtoIndex( proto)) {
  case PROTO_ALL:
   memset( &(c->protoMask), state ? 0xFF : 0, sizeof(c->protoMask));
   return 0;

  case PROTO_IP:
  case PROTO_IPX:
   if( state)
    *pInt |= 1 << i-1;
   else
    *pInt &= ~(1 << i-1);
    
   return 0;
   
  default:
   return -1; 
 }
}

