/* multiplex.cpp -- Multiplex chains + compression wrapper-routines

   This file is part of the SIB - simple IPX bridge

   Copyright (C) 2001 Szomraky Stefan

   The SIB is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of
   the License, or (at your option) any later version.

   The SIB is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with the SIB; see the file COPYING.
   If not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
   
   Szomraky Stefan
   stsz@gmx.net
 */
#include "multiplex.h"

MPFrame **MPFrames = NULL;

int enMultiplex( u_char *frame, u_int len)
{
 static u_char buf[MAX_BUF_LEN];
 u_int newlen = len+1;
 bool override = false;
 
 // send a "standard" frame
 buf[0] = 0;
 
 // insert CONTROL BYTE (for later implementations)

 if( config.customFilter) {
  switch( config.customFilter( frame, len)) {
   case 1:
    return 1;
   
   case 0:
    break;

   case -1:
    override = true;
    break;
  }
 } 
 
 if( !override) {
   if( !config.noMACFilter) {
     if( !addLocal( frame+6)) 
      printf( "new local MAC: %s\n", btoMAC( frame+6));
   
     if( isLocal( frame)) return 1;
   }
   
   if( filterFrame( frame, len)) return 1;
  
   if( config.noIPXSMB) if( isIPXSMB( frame, len)) return 1;
 }
 
 if( config.doCompression) {
  // NEVER fails (according to LZO docs) 
  // using buf+1 the compressed frame does not overwrite the CONTROL byte
#ifdef HAVE_LZO
  lzo1x_1_compress( frame, len, buf+1, &newlen, lzo_wrkmem);
#endif  
  // mark frame as compressed 
  if( newlen < len) {
    if( config.debug) printf( "better to send compressed frame\n");
    buf[0] |= MSG_LZOFRAME;
  }    
  else {
   if( config.debug) printf( "better to send uncompressed frame\n");
   memcpy( buf+1, frame, len);
   newlen = len;
  }
   
  // compress overwrites newlen 
  newlen++;
 }
 else
  memcpy( buf+1, frame, len);
 
 if( config.debug) printf( "ETH (%u) -> UDP (%u)\n", len, newlen);
 
 // to avoid negative compression stats newlen-1 is used (CONTROL byte)
 if( config.statsInterval > 0) newFrame( STAT_SENT, len, newlen-1);
 return sendFrameUDP( buf, newlen);
}

int deMultiplex( u_char *frame, u_int len)
{
 u_int newlen = len-1;
 static u_char buf[MAX_BUF_LEN];
 if( len == 0) return 1;
 
 // if control byte is not 0, this is a special packet 
 // NOTE: MSG_SPLIT and MSG_LZOFRAME are mutually exclusive
 if( frame[0] != 0) {
  if( frame[0] == MSG_SPLIT) return handleMultipacket( frame, len);
  if( frame[0] == MSG_LZOFRAME) {
#ifdef HAVE_LZO  
   lzo1x_decompress( frame+1, len-1, buf, &newlen, NULL);
   if( config.debug) printf( "received compressed frame\n");
#else
   if( config.debug) printf( "WARNING: compressed frame received, but no LZO support built in\n");   
#endif
  }
  else
   return 1; 
 }
 else { 
  //ignore CONTROL BYTE
  // (newlen is already set) newlen = len-1;
  memcpy( buf, frame+1, newlen);
 }     

 if( config.debug) printf( "UDP (%u) -> ETH (%u)\n", len, newlen);
 // len-1 for control byte compensation
 if( config.statsInterval > 0) newFrame( STAT_RECV, newlen, len-1);
  
 return sendFrameETH( buf, newlen);
}


int handleMultipacket( u_char *frame, u_int len)
{
 int i;
 int seq = frame[1];
 int part = frame[2], parts = frame[3];

 if( config.debug)
  printf( "received multipacket. seq: %u, part: %u, sum_parts: %u\n", (unsigned)seq, (unsigned)part, (unsigned)parts);
 
 if( MPFrames == NULL) {
  if( config.debug >= 3) printf( "MPFrames == NULL, allocating\n");
  
  MPFrames = new MPFrame*[256];
  for( i = 0; i < 256; i++) MPFrames[i] = NULL;
 } 
 
 if( MPFrames[seq] == NULL) {
  if( config.debug >= 3) printf( "MPFrames[%u] == NULL, allocating\n", seq);
  
  MPFrames[seq] = new MPFrame;
  MPFrames[seq]->parts = parts;
 }
  
 if( MPFrames[seq]->data == NULL) {
  if( config.debug >= 3) printf( "MPFrames[%u]->data == NULL, allocating\n", seq);

  MPFrames[seq]->data = new u_char*[parts];
  MPFrames[seq]->dlen = new u_int[parts];

  for( i = 0; i < parts; i++) {
   MPFrames[seq]->data[i] = NULL;
   MPFrames[seq]->dlen[i] = 0;
  } 
 }
 
 if( MPFrames[seq]->data[part] == NULL) { 
  if( config.debug >= 3) printf( "MPFrames[%u]->data[%u] == NULL, allocating\n", seq, part);
  
  MPFrames[seq]->data[part] = new u_char[len-4];
  memcpy( MPFrames[seq]->data[part], frame+4, len-4);
  MPFrames[seq]->dlen[part] = len-4;
  
  u_char buf[MAX_BUF_LEN];
  u_int bufcur = 0;
  
  for( i = 0; i < parts; i++) {
   if( !(MPFrames[seq]->data[i])) return 0;
   memcpy( buf+bufcur, MPFrames[seq]->data[i], MPFrames[seq]->dlen[i]);
   bufcur += MPFrames[seq]->dlen[i];
  }
  
  delete MPFrames[seq];
  MPFrames[seq] = NULL;
  
  if( config.debug) printf( "decomposing multipackage, size: %u\n", bufcur);
 
  return deMultiplex( buf, bufcur);
 }

 delete MPFrames[seq];
 MPFrames[seq] = NULL;

 return handleMultipacket( frame, len);
}
